"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sequence = void 0;
const files_1 = require("../files");
const log_1 = require("../log");
var SequenceStatus;
(function (SequenceStatus) {
    SequenceStatus[SequenceStatus["IDLE"] = 0] = "IDLE";
    SequenceStatus[SequenceStatus["RUNNING"] = 1] = "RUNNING";
    SequenceStatus[SequenceStatus["PAUSED"] = 2] = "PAUSED";
})(SequenceStatus || (SequenceStatus = {}));
class Sequence {
    constructor(camera, fd, display, ffprobe, send) {
        this.current = null;
        this.info = null;
        this.images = [];
        this.running = false;
        this.paused = false;
        this.progress = 0;
        this.frame = 0;
        this.frames = 0;
        this.exposure = 1000;
        this.log = (0, log_1.createLog)('seq');
        this.camera = camera;
        this.fd = fd;
        this.display = display;
        this.ffprobe = ffprobe;
        this.send = send;
    }
    start() {
        this.running = true;
    }
    stop() {
        this.running = false;
    }
    isRunning() {
        return this.running;
    }
    async load(seq) {
        this.current = seq;
        this.frame = 0;
        this.progress = 0;
        await this.enumerate();
        this.updateClients();
    }
    updateClients() {
        if (this.current !== null) {
            this.send({ cmd: 'select', state: this.getState() });
        }
    }
    async enumerate() {
        let screen;
        if (this.current === null) {
            this.log.error('Cannot enumerate sequence because it is not set');
            return;
        }
        try {
            this.images = await files_1.Files.enumerateSequence(this.current.path);
        }
        catch (err) {
            this.log.error(`Error enumerating images in sequence: ${this.current.name}`, err);
            return;
        }
        this.frames = this.images.length;
        this.log.info(`Sequence ${this.current.name} contains ${this.images.length} image${this.images.length === 1 ? '' : 's'}`);
        if (this.frames > 0) {
            this.info = await this.ffprobe.info(this.images[0].path);
        }
        if (this.info !== null) {
            screen = this.display.getScreen();
            this.display.setSource(this.info.width, this.info.height);
            this.log.info(`Screen   : ${screen.width},${screen.height}`);
            this.log.info(`Sequence : ${this.info.width},${this.info.height}`);
            this.log.info(`Display  : ${JSON.stringify(this.display.getDimensions())}`);
        }
    }
    unload() {
        this.current = null;
        this.info = null;
        this.images = [];
    }
    getState() {
        const dimensions = this.display.getDimensions();
        const source = this.display.getSource();
        const screen = this.display.getScreen();
        return {
            display: {
                width: dimensions.w,
                height: dimensions.h
            },
            offset: {
                x: dimensions.x,
                y: dimensions.y
            },
            screen,
            source,
            sequence: {
                hash: this.current.hash,
                name: this.current.name,
                progress: this.progress,
                current: this.frame,
                frames: this.frames,
                status: this.getStatus()
            },
            exposure: this.exposure
        };
    }
    getSequenceState() {
        if (this.current === null) {
            return null;
        }
        return {
            hash: this.current.hash,
            name: this.current.name,
            progress: this.progress
        };
    }
    setExposure(ms) {
        this.exposure = ms;
    }
    getStatus() {
        if (this.running && this.paused) {
            return SequenceStatus.PAUSED;
        }
        else if (this.running && !this.paused) {
            return SequenceStatus.RUNNING;
        }
        return SequenceStatus.IDLE;
    }
    getCurrent() {
        if (this.current !== null && this.images.length > 0 && typeof this.images[this.frame] !== 'undefined') {
            return this.images[this.frame];
        }
        return null;
    }
}
exports.Sequence = Sequence;
//# sourceMappingURL=index.js.map