import { Files } from '../files';
import { createLog } from '../log'
import type { Logger } from 'winston';
import type { SequenceObject, ImageObject } from '../files';
import type { FD, fdOutgoingPosition } from '../fd';
import type { Camera } from '../camera';
import type { Display, Dimensions } from '../display';
import type { FFPROBE, VideoInfo } from '../ffprobe';

enum SequenceStatus {
  IDLE,
  RUNNING,
  PAUSED
}

export class Sequence {
	private log : Logger;
	private current : SequenceObject = null;
	private info : VideoInfo = null;
	private images : ImageObject[] = [];
	private camera : Camera;
	private display : Display;
	private ffprobe : FFPROBE;
	private fd : FD;
	private send : Function;
	private running : boolean = false;
	private paused : boolean = false;

	private progress : number = 0;
	private frame : number = 0;
	private frames : number = 0;
	
	private exposure : number = 1000;

	constructor (camera : Camera, fd : FD, display: Display, ffprobe : FFPROBE, send : Function) {
		this.log = createLog('seq');
		this.camera = camera;
		this.fd = fd;
		this.display = display;
		this.ffprobe = ffprobe;
		this.send = send;
	}

	public start () {
		this.running = true;
	}

	public stop () {
		this.running = false;
	}

	public isRunning () {
		return this.running;
	}

	public async load (seq : SequenceObject) {
		this.current = seq;
		this.frame = 0;
		this.progress = 0;
		await this.enumerate();
		this.updateClients();
	}

	public updateClients () {
		if (this.current !== null) {
			this.send({ cmd : 'select', state : this.getState() });
		}
	}

	private async enumerate () {
		let screen : Dimensions;

		if (this.current === null)  {
			this.log.error('Cannot enumerate sequence because it is not set');
			return;
		}

		try {
			this.images = await Files.enumerateSequence(this.current.path);
		} catch (err) {
			this.log.error(`Error enumerating images in sequence: ${this.current.name}`, err);
			return;
		}

		this.frames = this.images.length;

		this.log.info(`Sequence ${this.current.name} contains ${this.images.length} image${this.images.length === 1 ? '' : 's'}`);

		if (this.frames > 0) {
			this.info = await this.ffprobe.info(this.images[0].path);
		}
		if (this.info !== null) {
			screen = this.display.getScreen();
			this.display.setSource(this.info.width, this.info.height);
			this.log.info(`Screen   : ${screen.width},${screen.height}`);
			this.log.info(`Sequence : ${this.info.width},${this.info.height}`);
			this.log.info(`Display  : ${JSON.stringify(this.display.getDimensions())}`)
		}
	}

	public unload () {
		this.current = null;
		this.info = null;
		this.images = [];
	}

	public getState () : State {
		const dimensions : fdOutgoingPosition = this.display.getDimensions();
		const source : Dimensions = this.display.getSource();
		const screen : Dimensions = this.display.getScreen();
		return {
			display : {
				width : dimensions.w,
				height : dimensions.h
			},
			offset : {
				x : dimensions.x,
				y : dimensions.y
			},
			screen,
			source,
			sequence : {
				hash : this.current.hash,
				name : this.current.name,
				progress : this.progress,
				current : this.frame,
				frames : this.frames,
				status : this.getStatus() as SequenceStatus
			},
			exposure : this.exposure
		}
	}

	public getSequenceState () : SequenceState {
		if (this.current === null) {
			return null;
		}
		return {
			hash : this.current.hash,
			name : this.current.name,
			progress : this.progress
		}
	}

	public setExposure (ms : number) {
		this.exposure = ms;
	}

	public getStatus () : SequenceStatus {
		if (this.running && this.paused) {
			return SequenceStatus.PAUSED;
		} else if (this.running && !this.paused) {
			return SequenceStatus.RUNNING;
		}
		return SequenceStatus.IDLE;
	}

	public getCurrent () : ImageObject {
		if (this.current !== null && this.images.length > 0 && typeof this.images[this.frame] !== 'undefined') {
			return this.images[this.frame]
		}
		return null;
	}
}